#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo logger)
  #:use-module (oop goops)
  #:use-module ((srfi srfi-1) #:select (take-while member))
  #:use-module ((srfi srfi-26) #:select (cut))
  #:use-module (logging logger)
  #:use-module (logging port-log)
  #:export (setup-logging!
            shutdown-logging!
            valid-log-level?)
  #:re-export (log-msg))

(define log-levels '(DEBUG INFO NOTICE WARN ERROR))

(define (valid-log-level? level)
  (member level log-levels eq?))

(define* (setup-logging! #:key (level 'INFO))
  (let ((logger (make <logger>))
        (handler (make <port-log> #:port (current-error-port))))
    (for-each (cut disable-log-level! handler <>)
              (take-while (negate (cut equal? level <>)) log-levels))
    (add-handler! logger handler)
    (set-default-logger! logger)
    (open-log! logger)))

(define (shutdown-logging!)
  (flush-log)  ; since no args, it uses the default
  (close-log!) ; ditto
  (set-default-logger! #f))
